import curses
from Queue import Queue
from time import strftime

class BaseWindow(object):
    def __init__(self, main, top, left, height, width):
        # Reference to the main window
        self.main = main

        # Window position
        self.top    = top
        self.left   = left
        self.height = height
        self.width  = width

        self.lines = []
        self.line_queue = Queue()

        # Initialise window
        self.window = curses.newwin(height, width, top, left)

    def clear(self):
        self.window.clear()
        self.refresh()

    def redraw(self):
        self.window.clear()

        while not self.line_queue.empty():
            self.lines.append(self.line_queue.get())

        if hasattr(self,'displayed_help') and self.displayed_help:
            return

        for y, line in enumerate(self.lines[-self.height:]):
            try:
                self.window.addstr(y, 0, line.ljust(self.width))
            except curses.error:
                pass
        self.refresh()

    def refresh(self):
        self.window.refresh()

    def append(self, msg):
        time_prefix = strftime('%H:%M') + ' '
        while msg:
            msg = time_prefix + msg
            self.line_queue.put(msg[:self.width])
            msg = msg[self.width:]
        self.redraw()

